void pwm2_setup(unsigned long pwm_cprd, unsigned long pwm_dt, bool *synch_pwm, bool *pwm_event)
{
  bool centred = 1; // Setup center aligned PWM
  bool PWMmode = 0; // Setup unipolar PWM
  
  // Enable clocks in PMC
  pmc_set_writeprotect(0);
  pmc_enable_periph_clk(ID_PWM);
  
  if (*synch_pwm) {
 //Configure PWM channels 0,1,2,3 (PWML0,PWMH0,PWML1,PWMH1,PWML2,PWMH2,PWML3,PWMH3), (port C.2,C.3,C.4,C.5,C.6,C.7,C.8,C.9), (pins P34,P35,P36,P37,P38,P39,P40,P41)
      REG_PIOC_PDR |= PIO_PER_P2 | PIO_PER_P3;  // PIOC port 2,3 Disable for PWM ch 0
      REG_PIOC_PDR |= PIO_PER_P4 | PIO_PER_P5;  // PIOC port 4,5 Disable for PWM ch 1
      REG_PIOC_PDR |= PIO_PER_P6 | PIO_PER_P7;  // PIOC port 6,7 Disable for PWM ch 2
      REG_PIOC_PDR |= PIO_PER_P8 | PIO_PER_P9;  // PIOC port 8,9 Disable for PWM ch 3
      REG_PIOC_ABSR |= PIO_PER_P2 | PIO_PER_P3; // Peripheral B Select for PWM ch 0
      REG_PIOC_ABSR |= PIO_PER_P4 | PIO_PER_P5; // Peripheral B Select for PWM ch 1
      REG_PIOC_ABSR |= PIO_PER_P6 | PIO_PER_P7; // Peripheral B Select for PWM ch 2
      REG_PIOC_ABSR |= PIO_PER_P8 | PIO_PER_P9; // Peripheral B Select for PWM ch 3
      REG_PWM_SCM  = 0x01000F; // Setup channels 0,1,2,3 as synchronous channels mode 1
  } else {
    //Configure PWM channels 2,3 (PWML2,PWMH2,PWML3,PWMH3), (port C.6,C.7,C.8,C.9), (pins P38,P39,P40,P41)
      REG_PIOC_PDR |= PIO_PER_P6 | PIO_PER_P7;  // PIOC port 6,7 Disable for PWM ch 2
      REG_PIOC_PDR |= PIO_PER_P8 | PIO_PER_P9;  // PIOC port 8,9 Disable for PWM ch 3
      REG_PIOC_ABSR |= PIO_PER_P6 | PIO_PER_P7; // Peripheral B Select for PWM ch 2
      REG_PIOC_ABSR |= PIO_PER_P8 | PIO_PER_P9; // Peripheral B Select for PWM ch 3
      REG_PWM_SCM  = 0x000000; // Setup channels 0,1,2,3 as asynchronous channels
  }
  if (centred & PWMmode){ // pwmclk=Mclk/8, center aligned, dead time enabled, pwm ch3 inverted
    REG_PWM_CMR2 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_CALG | PWM_CMR_DTE;
    REG_PWM_CMR3 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_CALG | PWM_CMR_CPOL | PWM_CMR_DTE;
  } else if (PWMmode){// pwmclk=Mclk/8, left aligned, dead time enabled, pwm ch3 inverted
    REG_PWM_CMR2 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_DTE;
    REG_PWM_CMR3 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_CPOL | PWM_CMR_DTE;
  } else if (centred){// pwmclk=Mclk/8, center aligned, dead time enabled
    REG_PWM_CMR2 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_CALG | PWM_CMR_DTE;
    REG_PWM_CMR3 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_CALG | PWM_CMR_DTE;
  } else {// pwmclk=Mclk/8, left aligned, dead time enabled
    REG_PWM_CMR2 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_DTE;
    REG_PWM_CMR3 = PWM_CMR_CPRE_MCK_DIV_8 | PWM_CMR_DTE;
  }
  REG_PWM_DT2 = pwm_dt; //Channel 2 Dead Time Register for outputs PWML2,PWMH2
  REG_PWM_DT3 = pwm_dt; //Channel 3 Dead Time Register for outputs PWML3,PWMH3

  REG_PWM_CPRD2 = pwm_cprd; //Channel 2 Period Register
  REG_PWM_CPRD3 = pwm_cprd; //Channel 3 Period Register

  REG_PWM_CDTY2 = pwm_cprd;
  REG_PWM_CDTY3 = pwm_cprd;
  
  REG_PWM_CMPV0 = 0;
  
  if (*pwm_event) {
      unsigned long cmpv = 0.1*pwm_cprd;
      REG_PWM_CMPV0 = PWM_CMPV_CV(cmpv);
      REG_PWM_CMPM0 = PWM_CMPM_CEN;
  // REG_PWM_CMPVUPD0 = (pwm_cprd/2);
  // REG_PWM_CMPMUPD0 = 1;
      PWM->PWM_ELMR[0] = PWM_ELMR_CSEL0; // Event line 0, enable comparison 0
   //   REG_PWM_IER2 = 0x0100;
  } else {
      REG_PWM_CMPM0 = 0;
  }

  REG_PWM_SCUP = 0; // Channel update period
  REG_PWM_OS   = 0;

  if (*synch_pwm) {
    REG_PWM_ENA =  PWM_ENA_CHID0 | PWM_ENA_CHID1 | PWM_ENA_CHID2 | PWM_ENA_CHID3; //PWM activate channel 0,1,2,3
  } else {
    REG_PWM_ENA =  PWM_ENA_CHID2 | PWM_ENA_CHID3; //PWM activate channel 2,3
  }
}

void pwm_override_set(unsigned short Amde, unsigned short Bmde)
{
  if ((Amde==3) & (Bmde==3)) {
      REG_PWM_OSC = 0x0C000C; // Output 2H,2L,3H,3L override off
  } else if ((Bmde==0) & (Amde==1)) {
      REG_PWM_OSC = 0x000004; // Output 2H override off
      REG_PWM_OSS = 0x0C0008; // Output 3H,3L,2L override on
  } else if ((Bmde==0) & (Amde==2)) {
      REG_PWM_OSC = 0x040000; // Output 2L override off
      REG_PWM_OSS = 0x08000C; // Output 3H,3L,2H override on
  } else if ((Bmde==0) & (Amde==3)) {
      REG_PWM_OSC = 0x040004; // Output 2L,2H override off
      REG_PWM_OSS = 0x080008; // Output 3H,3L override on
  } else if ((Amde==0) & (Bmde==1)) {
      REG_PWM_OSC = 0x000008; // Output 3H override off
      REG_PWM_OSS = 0x0C0004; // Output 3L,2H,2L override on
  } else if ((Amde==0) & (Bmde==2)) {
      REG_PWM_OSC = 0x080000; // Output 3L override off
      REG_PWM_OSS = 0x04000C; // Output 3H,2H,2L override on
  } else if ((Amde==0) & (Bmde==3)) {
      REG_PWM_OSC = 0x080008; // Output 3L,3H override off
      REG_PWM_OSS = 0x040004; // Output 2H,2L override on
  }
}
//Set the PWM duty-cycle and block a phase or high-side output
void pwmc2_duty_sync(unsigned long pwm_cprd, bool *synch_pwm, double *Dty, unsigned short *Mde, bool Boot_charge, bool Block, bool *pwm_event)
{
//  unsigned long start_time = micros();
  double Adty = *(Dty+0);
  double Bdty = *(Dty+1);
  unsigned short Amde = *(Mde+0);
  unsigned short Bmde = *(Mde+1);
  
  REG_PWM_OOV   = 0;
  REG_PWM_CDTY0 = pwm_cprd;
  if (*pwm_event) {
      unsigned long cmpv = 0.5*Adty*pwm_cprd;
      REG_PWM_CMPVUPD0 = PWM_CMPV_CV(cmpv);
      REG_PWM_CMPMUPD0 = PWM_CMPM_CEN;
  // REG_PWM_CMPVUPD0 = (pwm_cprd/2);
  // REG_PWM_CMPMUPD0 = 1;
   //   PWM->PWM_ELMR[0] = PWM_ELMR_CSEL0; // Event line 0, enable comparison 0
   //   REG_PWM_IER2 = 0x0100;
  } else {
      REG_PWM_CMPMUPD0 = 0;
  }
  if (Block) {
    REG_PWM_CPRD0 = pwm_cprd;
    REG_PWM_CPRD2 = pwm_cprd; // Channel period
    REG_PWM_CPRD3 = pwm_cprd; // Channel period
    REG_PWM_CDTY2 = pwm_cprd;
    REG_PWM_CDTY3 = pwm_cprd;
    REG_PWM_OSS  = 0x0C000C; // Output 2,3 override on
  } else if (Boot_charge) {
    REG_PWM_CPRD0 = pwm_cprd;
    REG_PWM_CPRD2 = pwm_cprd; // Channel period
    REG_PWM_CPRD3 = pwm_cprd; // Channel period
    REG_PWM_CDTY2 = pwm_cprd;
    REG_PWM_CDTY3 = pwm_cprd;
    pwm_override_set(Amde,Bmde);
  } else if ((*synch_pwm == 1) & ((REG_PWM_ISR2 & 1) ==1)) { // WRDY=1
      REG_PWM_CPRDUPD0 = pwm_cprd;
      pwm_override_set(Amde,Bmde);
      REG_PWM_CDTYUPD2 = (pwm_cprd-Adty*pwm_cprd);;
      REG_PWM_CDTYUPD3 = (pwm_cprd-Bdty*pwm_cprd);
      REG_PWM_SCUC = 1; // synch channels update unlock
  } else if ((*synch_pwm == 0)) {
      REG_PWM_CPRD2 = pwm_cprd; // Channel period
      REG_PWM_CPRD3 = pwm_cprd; // Channel period
      pwm_override_set(Amde,Bmde);
      REG_PWM_CDTY2 = (pwm_cprd-Adty*pwm_cprd);
      REG_PWM_CDTY3 = (pwm_cprd-Bdty*pwm_cprd);
  }
}
